/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "pagamento.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include "cmd.h"
#include "list.h"
#include "util.h"
#include "totais.h"

static int pagamento_totais_update(struct declaracao *dec, struct pagamento *pagamento)
{
	int r;
	r = totais_add(dec, "PAGAMENTOS", pagamento->pagamento);
	r += totais_add(dec, "REEMBOLSOS", pagamento->reembolso);
	if (pagamento->dependente) {
		r += totais_add(dec, "PAGAMENTOSDEP", pagamento->pagamento);
		r += totais_add(dec, "REEMBOLSOSDEP", pagamento->reembolso);
	} else {
		r += totais_add(dec, "PAGAMENTOSTIT", pagamento->pagamento);
		r += totais_add(dec, "REEMBOLSOSTIT", pagamento->reembolso);
	}
	switch (pagamento->codigo) {
	case 10:
	case 12:
	case 21:
	case 26:
		r += totais_add(dec, "MEDICAS", pagamento->pagamento - pagamento->reembolso);
		break;
	case 36:
		r += totais_add(dec, "PREVIDENCIA", pagamento->pagamento - pagamento->reembolso);
		break;
	}
	return r;
}

void pagamento_free(void *pointer)
{
	struct pagamento *pagamento = pointer;
	if (pagamento->cnpj)
		free(pagamento->cnpj);
	if (pagamento->nome)
		free(pagamento->nome);
	free(pagamento);
}

static int pagamento_cmp(void *p1, void *p2)
{
	struct pagamento *r1 = p1;
	struct pagamento *r2 = p2;
	/* O pagamento maior vem primeiro. */
	if (r1->pagamento > r2->pagamento)
		return -1;
	else if (r1->pagamento < r2->pagamento)
		return 1;
	return 0;
}

static struct pagamento * pagamento_new(char **args, int argc)
{
	struct pagamento *pagamento;
	int r = 0;
	pagamento = malloc(sizeof(*pagamento));
	r += set_int(args[1], &pagamento->codigo);
	r += set_string(args[2], &pagamento->cnpj);
	r += set_string(args[3], &pagamento->nome);
	r += set_llong(args[4], &pagamento->pagamento);
	r += set_llong(args[5], &pagamento->reembolso);
	if (r < 0 || pagamento->codigo < 0 ||
	    pagamento->pagamento < 0 || pagamento->reembolso < 0) {
		pagamento_free(pagamento);
		return NULL;
	}
	if (argc == 7) {
		r = set_int(args[6], &pagamento->dependente);
	} else {
		pagamento->dependente = 0;
	}
	if (r < 0 || pagamento->dependente < 0) {
		pagamento_free(pagamento);
		return NULL;
	}
	return pagamento;
}

static int run_pagamento(struct declaracao *dec, char **args, int argc)
{
	struct pagamento *pagamento;
	int r;
	if (argc != 6 && argc != 7)
		return -EINVAL;
	pagamento = pagamento_new(args, argc);
	if (!pagamento)
		return -ENOMEM;
	if (pagamento->dependente > list_size(dec->dependentes)) {
		pagamento_free(pagamento);
		return -EINVAL;
	}
	r = list_insert_ordered(&dec->pagamentos, pagamento, pagamento_cmp);
	if (r < 0) {
		pagamento_free(pagamento);
		return r;
	}
	r = pagamento_totais_update(dec, pagamento);
	if (r) {
		pagamento_free(pagamento);
		return r;
	}
	return 0;
}

void pagamento_salva(struct declaracao *dec, FILE *f)
{
	int i;
	struct pagamento *j;
	for (i = 0; j = list_get(dec->pagamentos, i); i++)
		fprintf(f, "pagamento %d \"%s\" \"%s\" %lld %lld %d\n",
			j->codigo, j->cnpj, j->nome, j->pagamento, j->reembolso, j->dependente);
}

static struct cmd cmd_pagamento = {
	.name = "pagamento",
	.run = run_pagamento,
};

int pagamento_cmd_init(void)
{
	cmd_add(&cmd_pagamento);
	return 0;
}

char * pagamento_cnpj_ordenado_cond(struct declaracao *dec,
				    int (*cond)(struct pagamento *), int n)
{
	struct pagamento *pagamento;
	int i;
	int j = 0;
	for (i = 0; (pagamento = list_get(dec->pagamentos, i)); i++) {
		if (cond(pagamento) && j++ == n)
			break;
	}
	if (!pagamento)
		return "";
	return pagamento->cnpj;
}

static int always(struct pagamento *pagamento)
{
	return pagamento != NULL;
}

char * pagamento_cnpj_ordenado(struct declaracao *dec, int n)
{
	return pagamento_cnpj_ordenado_cond(dec, always, n);
}

static int pagamento_medico(struct pagamento *pagamento)
{
	switch (pagamento->codigo) {
	case 10:
	case 12:
	case 21:
	case 26:
		return 1;
	default:
		return 0;
	}
	return 0;
}

int pagamento_instrucao(struct pagamento *pagamento)
{
	switch (pagamento->codigo) {
	case 1:
		return 1;
	default:
		return 0;
	}
	return 0;
}

char * medico_cnpj_ordenado(struct declaracao *dec, int n)
{
	return pagamento_cnpj_ordenado_cond(dec, pagamento_medico, n);
}

static int pagamento_inss(struct pagamento *pagamento)
{
	return pagamento->codigo == 36;
}

char *inss_cnpj_ordenado(struct declaracao *dec, int n)
{
	return pagamento_cnpj_ordenado_cond(dec, pagamento_inss, n);
}
